@@ -1,7 +1,18 @@
 #!/bin/bash
 
-# directory where libQt5XXX.so files can be found
-LIBDIR=/usr/lib/x86_64-linux-gnu
+# Deployment script for Linux x64 portable binary. This
+# script is tested on a clean Ubuntu 18.04, after running:
+# 
+# apt-get -y install \
+#  flex bison qt5-default libpoppler-dev \
+#  libpoppler-qt5-dev
+#
+
+# system library directory
+SYSLIBDIR=/lib/x86_64-linux-gnu
+
+# userspace library directory
+LIBDIR=/usr$SYSLIBDIR
 
 # directory where Qt plugins can be found
 PLUGINDIR=$LIBDIR/qt5/plugins
@@ -15,7 +26,7 @@ mkdir -p plugins
 
 # add README file
 cat > README << 'EOF'
-This is a portable version of TikZiT 2.0. To launch TikZiT, simply run
+This is a portable version of TikZiT 2.1. To launch TikZiT, simply run
 'bin/tikzit'. To install launcher and icons for the current user, make
 sure the 'bin' sub-directory is in your $PATH and run:
 
@@ -84,6 +95,16 @@ cp --no-dereference $LIBDIR/libcrypto.so* lib
 cp --no-dereference $LIBDIR/libpoppler.so* lib
 cp --no-dereference $LIBDIR/libpoppler-qt5.so* lib
 
+# add dependencies of poppler that are on Ubuntu 18.04,
+# but maybe not other distros...
+cp --no-dereference $LIBDIR/libgraphite2.so.3* lib
+cp --no-dereference $LIBDIR/libharfbuzz.so.0* lib
+cp --no-dereference $LIBDIR/libjbig.so.0* lib
+cp --no-dereference $LIBDIR/libjpeg.so.8* lib
+cp --no-dereference $LIBDIR/liblcms2.so.2* lib
+cp --no-dereference $LIBDIR/libpng16.so.16* lib
+cp --no-dereference $SYSLIBDIR/libz.so.1* lib
+
 # add Qt plugins used by TikZiT
 cp -R $PLUGINDIR/platforms plugins
 cp -R $PLUGINDIR/imageformats plugins
