@@ -75,17 +75,19 @@ public:
     void extendSelectionLeft();
     void extendSelectionRight();
 
+    void mergeNodes();
+
     void reorderSelection(bool toFront);
 
     void reverseSelectedEdges();
 
 
-	void getSelection(QSet<Node*> &selNodes, QSet<Edge*> &selEdges);
-	QSet<Node*> getSelectedNodes();
+    void getSelection(QSet<Node*> &selNodes, QSet<Edge*> &selEdges) const;
+    QSet<Node*> getSelectedNodes() const;
+
     void refreshSceneBounds();
 
     bool highlightHeads() const;
-
     bool highlightTails() const;
 
 public slots:
